//============================================================================
// File: U08232.C
// Func: RS232 Implementation Module for USB08 Demo Application
// Ver.: 1.01
// Auth: (C)2000,2001 by Oliver Thamm, MCT Elektronikladen GbR
//       http://hc08web.de/usb08
// Rem.: View/Edit this File with TAB-Size=4
//============================================================================

#include "settings.h"
#include "hc08jb8.h"			// HC08JB8 Register Definitions
#include "u08232.h"				// Header File for RS232 Module

//----------------------------------------------------------------------------
// Hardware Dependencies - Physical Port Usage for Software SCI Tx/Rx
// Change the following Definitions to meet your Hardware Requirements:
//----------------------------------------------------------------------------

// Transmit Line is PTC[0]

#define	setTxLow()	(PTC &= ~0x01)
#define setTxHigh()	(PTC |= 0x01)
#define enaTxOut()	(DDRC |= 0x01)

// Receive Line is PTA[7]

#define tstRxLvl()	(PTA & 0x80)
#define enaRxIn()	(DDRA &= ~0x80)

//----------------------------------------------------------------------------
// Hardware Dependencies - SSCI Bit Timing generated by System Timer TIM
// Change the following Definitions to meet your Hardware Requirements:
//----------------------------------------------------------------------------

// clear TSTOP Bit in TSC Register to activate Counter
// PS0..PS2 Prescaler Bits in TSC Register must be 0 (default)
// so the Counter Rate is 3 MHz (0.333s)

// 9600 Baud -> 104.1666 us per Bit -> 312.5 Clocks per Bit @ 3MHz
// 2400 Baud -> 416.6666 us per Bit -> 1250  Clocks per Bit @ 3MHz
// Adjust Value depending on Subroutine Call Overhead

//----------------------------------------------------------------------------

void delayHalfBit() {

	// subtract ~20 Clocks for Overhead!
	// 120 * 5 Clocks = 600 Clocks
	asm("\
		lda #120	\n\
__dhbl:	deca		\n\
		nop			\n\
		bne __dhbl	\n\
		");
	}


void delayBitTime() {
	delayHalfBit();
	delayHalfBit();
	}

//----------------------------------------------------------------------------

void initSSCI() {
	setTxHigh();				// set Output Data Latch H
	enaTxOut();					// enable Output Driver for Tx
	enaRxIn();					// Rx is an Input Line
	}

//----------------------------------------------------------------------------

void putSSCI(char c) {

	unsigned char n;
	unsigned char ccr_save;

//	ccr_save = getCCR();		// save current Interrupt Mask
//	disableINTR();				// disable Interrupts

	setTxLow();					// send Startbit
	delayBitTime();

	n=8;
	do {						// send 8 Databits, LSB first
		if((c&1)==0)
			setTxLow();
		else
			setTxHigh();
		delayBitTime();
		c >>= 1;
		} while(--n);

	setTxHigh();				// send Stopbit
	delayBitTime();
	delayBitTime();
//	setCCR(ccr_save);			// restore previous Interrupt Mask
	}

//----------------------------------------------------------------------------

char getSSCI() {

	char c;
	unsigned char n;
	unsigned char ccr_save;

//	ccr_save = getCCR();		// save current Interrupt Mask
//	disableINTR();				// disable Interrupts

	while(tstRxLvl()!=0) ;		// wait for H-L transition
	delayHalfBit();
	n=8;
	do {						// get 8 Databits
		delayBitTime();
		c >>= 1;
		if(tstRxLvl()!=0)
			c |= 0x80;
		} while(--n);
	delayBitTime();
	if(tstRxLvl()==0) {			// check Rx Line during Stopbit
		// add framing error
		// handling if desired
		}
	delayHalfBit();
//	setCCR(ccr_save);			// restore previous Interrupt Mask
	return c;
	}

//============================================================================

