//============================================================================
// File: U08MAIN.C
// Func: Main Module for USB08 Demo Application
// Ver.: 1.01
// Auth: (C)2000,2001 by Oliver Thamm, MCT Elektronikladen GbR
//       http://hc08web.de/usb08
// Make: Build the project using U08MAIN.C, U08KEY.C, U08ADC.C and VECJB8.C
//		 for Cosmic-C:
//           use CRTSJB8.S as C-Startup Module
//           use Linkfile USB08.LKF
//       for ImageCraft-C:
//           use Standard C-Startup Module
//           set Project Options: Target=JB8, Globals=Direct Page
// Rem.: View/Edit this File with TAB-Size=4
//============================================================================

//-- Includes ----------------------------------------------------------------

#include "settings.h"			// You must modify this file to select the
								// Interface (USB or RS232) and the Compiler!

#include "hc08jb8.h"			// HC908JB8 Register and Bitmap Definitions
#include "u08key.h"				// Keyboard Module Header File
#include "u08led.h"				// LED Module Header File (just Macros)
#include "u08adc.h"				// Soft ADC Module Header File

//-- some Assembly Level Stuff -----------------------------------------------

#define cli() 		asm("cli")
#define nop()		asm("nop")

//-- Data Type Definitions ---------------------------------------------------

typedef unsigned char uchar;

//-- Code Starts here --------------------------------------------------------

#ifdef USE_USB_PIPE
#include "u08usb.c"				// use USB implementation
#define initPipe	initUSB
#define getPipe  	getUSB
#define putPipe  	putUSB
#else
#include "u08232.c"				// use RS232 implementation
#define initPipe	initSSCI
#define getPipe  	getSSCI
#define putPipe  	putSSCI
#endif

//----------------------------------------------------------------------------

// Things that should be done immediately after Reset
// (this is called by the C-Startup Module)
//
void _HC08Setup() {
	CONFIG = 0x21;	// USB Reset Disable, COP Disable
	TSC    = 0x00;	// clear TSTOP, Prescaler=0
	}

//----------------------------------------------------------------------------

// Dummy Interrupt Handler
// Place a Breakpoint here in case you are looking for spurious Interrupts
//
#ifdef IMAGECRAFT_C
#pragma interrupt_handler isrDummy
#endif
#ifdef COSMIC_C
@interrupt
#endif
void isrDummy() {
	nop();						// just for Debugging
	}

//----------------------------------------------------------------------------

void main() {

	uchar n, a;
	uchar io_buffer[8];
	uchar adc[3];

	initPipe();		// init RS232 or USB Pipe
	initLED();		// init LED Output
	initKey();		// init Key Input
	initSADC();		// init Soft ADC

	cli();

	a = 0;
	while(1) {

		// update ADC results (1 out of 3 at one time)

		adc[a] = getSADC(a+1);
		if(++a==3) a=0;

		// get data from input pipe

		n=0;
		do {
			io_buffer[n++] = getPipe();
			} while(n<8);

		// process input data

		if(io_buffer[0]==0) offLED(1);
		else onLED(1);
		if(io_buffer[1]==0) offLED(2);
		else onLED(2);
		if(io_buffer[2]==0) offLED(3);
		else onLED(3);

		// send data to output pipe

		io_buffer[0] = getKey(1);
		io_buffer[1] = getKey(2);
		io_buffer[2] = getKey(3);
		io_buffer[3] = adc[0];
		io_buffer[4] = adc[1];
		io_buffer[5] = adc[2];

		n=0;
		do {
			putPipe(io_buffer[n++]);
			} while(n<8);

		}
	}

//============================================================================

